<?php
namespace WpAssetCleanUp;

/**
 * Class PluginsManager
 * @package WpAssetCleanUp
 */
class PluginsManager
{
    /**
     * @var array
     */
    public $data = array();

    // [wpacu_pro]
	/**
	 * PluginsManager constructor.
	 */
	public function __construct()
    {
        // Note: The rules update takes place in /pro/classes/UpdatePro.php
	    if (Misc::getVar('get', 'page') === WPACU_PLUGIN_ID . '_plugins_manager') {
		    add_action('wpacu_admin_notices', array($this, 'notices'));
	    }
    }
    // [/wpacu_pro]

	/**
	 *
	 */
	public function page()
    {
    	// Get active plugins and their basic information
	    $this->data['active_plugins'] = self::getActivePlugins();
	    $this->data['plugins_icons']  = Misc::getAllActivePluginsIcons();

	    // [wpacu_pro]
	    $this->data['rules']          = self::getAllRules(); // get all rules from the database (for either the frontend or dash view)

	    $this->data['mu_file_missing']  = false; // default
	    $this->data['mu_file_rel_path'] = '/' . str_replace(Misc::getWpRootDirPath(), '', WPMU_PLUGIN_DIR)
	                                      . '/' . \WpAssetCleanUpPro\PluginPro::$muPluginFileName;

	    if ( ! is_file(WPMU_PLUGIN_DIR . '/' . \WpAssetCleanUpPro\PluginPro::$muPluginFileName) ) {
			$this->data['mu_file_missing']  = true; // alert the user in the "Plugins Manager" area
	    }

	    $postTypes                     = get_post_types( array( 'public' => true ) );
	    $this->data['post_types_list'] = Misc::filterPostTypesList($postTypes);
        // [/wpacu_pro]

	    Main::instance()->parseTemplate('admin-page-plugins-manager', $this->data, true);
    }

    // [wpacu_pro]
	/**
	 * @param false $fetchAllLocations (if set to true, it will return the rules for both the frontend and the backend
	 *
	 * @return array
	 */
	public static function getAllRules($fetchAllLocations = false)
	{
		$pluginsRulesDbListJson = get_option(WPACU_PLUGIN_ID . '_global_data');

		if ($pluginsRulesDbListJson) {
			$pluginsRulesDbList = @json_decode($pluginsRulesDbListJson, true);

			// Issues with decoding the JSON file? Return an empty list
			if (Misc::jsonLastError() !== JSON_ERROR_NONE) {
				return array();
			}

			// 1) For listing them in "Overview"
			if ($fetchAllLocations) {
                $rulesList = array();

				if ( isset( $pluginsRulesDbList['plugins'] ) && ! empty( $pluginsRulesDbList['plugins'] ) ) {
					$rulesList['plugins'] = $pluginsRulesDbList['plugins'];
				}

				if ( isset( $pluginsRulesDbList['plugins_dash'] ) && ! empty( $pluginsRulesDbList['plugins_dash'] ) ) {
					$rulesList['plugins_dash'] = $pluginsRulesDbList['plugins_dash'];
				}

				return $rulesList;
            }

			// 2) For listing them within "Plugins Manager" -> "In Frontend View" or "In the Dashboard" when the admin is managing the rules
			$wpacuSubPage = ( isset($_GET['wpacu_sub_page']) && $_GET['wpacu_sub_page'] ) ? $_GET['wpacu_sub_page'] : 'manage_plugins_front';

			$mainGlobalKey = ($wpacuSubPage === 'manage_plugins_front') ? 'plugins' : 'plugins_dash';

			if ( isset( $pluginsRulesDbList[$mainGlobalKey] ) && ! empty( $pluginsRulesDbList[$mainGlobalKey] ) ) {
				return $pluginsRulesDbList[$mainGlobalKey];
			}
		}

		return array();
	}
    // [/wpacu_pro]

	/**
	 * @return array
	 */
	public static function getActivePlugins()
	{
		$activePluginsFinal = array();

		if ( isset($GLOBALS['wpacu_original_active_plugins']) && is_array($GLOBALS['wpacu_original_active_plugins']) && ! empty($GLOBALS['wpacu_original_active_plugins']) ) {
			$activePlugins = array();

			foreach ( $GLOBALS['wpacu_original_active_plugins'] as $plugin ) {
				if ( ! validate_file( $plugin )                     // $plugin must validate as file.
				     && '.php' === substr( $plugin, - 4 )             // $plugin must end with '.php'.
				     && file_exists( WP_PLUGIN_DIR . '/' . $plugin ) // $plugin must exist.
				) {
					$activePlugins[] = WP_PLUGIN_DIR . '/' . $plugin;
				}
			}
		} else {
			// Get active plugins and their basic information
			$activePlugins = wp_get_active_and_valid_plugins();
		}

		// Also check any network activated plugins in case we're dealing with a MultiSite setup
		if ( is_multisite() ) {
			$activeNetworkPlugins = wp_get_active_network_plugins();

			if ( ! empty( $activeNetworkPlugins ) ) {
				foreach ( $activeNetworkPlugins as $activeNetworkPlugin ) {
					$activePlugins[] = $activeNetworkPlugin;
				}
			}
		}

		$activePlugins = array_unique($activePlugins);

		foreach ($activePlugins as $pluginPath) {
			// Skip Asset CleanUp as it's obviously needed for the functionality
			if (strpos($pluginPath, 'wp-asset-clean-up') !== false) {
				continue;
			}

			$networkActivated = isset($activeNetworkPlugins) && in_array($pluginPath, $activeNetworkPlugins);

			$pluginRelPath = trim(str_replace(WP_PLUGIN_DIR, '', $pluginPath), '/');

			$pluginData = get_plugin_data($pluginPath);

			$activePluginsFinal[] = array(
                'title'             => $pluginData['Name'],
                'path'              => $pluginRelPath,
                'network_activated' => $networkActivated
			);
		}

		usort($activePluginsFinal, static function($a, $b)
		{
			return strcmp($a['title'], $b['title']);
		});

		return $activePluginsFinal;
	}

	// [wpacu_pro]
	/**
	 * Make sure there is a status for the rule, otherwise it's likely set to "Load it",
	 * thus the rule wouldn't count
	 * @param bool $checkIfPluginIsActive
	 * @param bool $getRulesForAllLocations
     *
	 * @return array
	 */
	public static function getPluginRulesFiltered($checkIfPluginIsActive = true, $getRulesForAllLocations = false)
    {
	    $pluginsWithRules = array();

		$pluginsAllDbRules = self::getAllRules($getRulesForAllLocations);

		// Are there any load exceptions / unload RegExes?
	    if (! empty( $pluginsAllDbRules ) ) {
	        foreach ($pluginsAllDbRules as $locationKey => $pluginsRules) {
		        foreach ( $pluginsRules as $pluginPath => $pluginData ) {
			        // Only the rules for the active plugins are retrieved
			        if ( $checkIfPluginIsActive && ! Misc::isPluginActive( $pluginPath ) ) {
				        continue;
			        }

			        // 'status' refers to the Unload Status (any option that was chosen)
			        $pluginStatus = isset( $pluginData['status'] ) && ! empty( $pluginData['status'] ) ? $pluginData['status'] : array();

			        if ( ! empty( $pluginStatus ) ) {
				        $pluginsWithRules[ $locationKey ][ $pluginPath ] = $pluginData;
			        }
		        }
	        }

		    }

	    return $pluginsWithRules;
    }

	/**
     * @param $ddKeyName
	 * @param $enabled
     * @param $postTypesList
	 * @param $currentPostTypes
	 * @param $pluginPath
	 */
	public static function buildPostTypesListDd($ddKeyName, $enabled, $postTypesList, $currentPostTypes, $pluginPath)
	{
		$ddList = array();

		foreach ($postTypesList as $postTypeKey => $postTypeValue) {
			if (in_array($postTypeKey, array('post', 'page', 'attachment'))) {
				$ddList['WordPress (default)'][$postTypeKey] = $postTypeValue;
			} else {
				$ddList['Custom Post Types (Singular pages)'][$postTypeKey] = $postTypeValue;
			}
		}
		?>
        <select multiple="multiple"
                style="width: 100%;"
                data-placeholder="<?php esc_attr_e('Choose the post types'); ?>..."
                class="<?php if ( $enabled && ! empty($currentPostTypes) ) { ?>wpacu_chosen_select<?php } ?> wpacu_plugin_manage_via_post_type_dd"
                name="wpacu_plugins[<?php echo esc_attr($pluginPath); ?>][<?php echo $ddKeyName; ?>][values][]">
			<?php
			foreach ($ddList as $groupLabel => $groupPostTypesList) {
				echo '<optgroup label="'.$groupLabel.'">';

				foreach ($groupPostTypesList as $postTypeKey => $postTypeValue) {
					?>
                    <option <?php if (in_array($postTypeKey, $currentPostTypes)) { echo 'selected="selected"'; } ?> value="<?php echo esc_attr($postTypeKey); ?>"><?php echo esc_html($postTypeValue); ?></option>
					<?php
				}

				echo '</optgroup>';
			}
			?>
        </select>
		<?php
	}

	/**
	 *
	 */
	public function notices()
	{
		// After "Save changes" is clicked
		if (get_transient('wpacu_plugins_manager_updated')) {
			delete_transient('wpacu_plugins_manager_updated');

			$appliedForText = '';
			if ( isset($_GET['wpacu_sub_page']) ) {
				if ( $_GET['wpacu_sub_page'] === 'manage_plugins_front' ) {
					$appliedForText = 'the frontend view';
				} elseif ( $_GET['wpacu_sub_page'] === 'manage_plugins_dash' ) {
					$appliedForText = 'the Dashboard view (/wp-admin/)';
				}
			}

			if ($appliedForText !== '') {
			?>
			<div style="margin-bottom: 15px; margin-left: 0; width: 90%;" class="notice notice-success is-dismissible">
				<p><span class="dashicons dashicons-yes"></span> <?php echo sprintf(__('The plugins\' rules were successfully applied within %s.', 'wp-asset-clean-up'), $appliedForText); ?></p>
			</div>
			<?php
            }
		}
	}
	// [/wpacu_pro]
}
